#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include "spi.h"
#include "lcd.h"

void bresenham_line (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);
void h_line (unsigned int x1, unsigned int x2, unsigned int y);
void v_line (unsigned int x, unsigned int y1, unsigned int y2);

void lcd_clear(void)
{
	spi_putc(Escape);
	spi_putc(11);
}

void lcd_writebyte(unsigned char byte)
{
	if (byte==Escape)
		spi_putc(Escape);
	spi_putc(byte);
}

void lcd_setpixel(unsigned int x, unsigned int y)
{
	if ((x>=640)||(y>=480))
		return;
	lcd_setadress(x/8, y);
	spi_putc(Escape);
	spi_putc(16|(x&7));
}

void lcd_writetext(unsigned char *text)
{
	while (*text)
	{	spi_putc(Escape);
		spi_putc(192);
		if (*text==Escape)
			spi_putc(Escape);
		spi_putc(*text);
		text++;
	}
}

void lcd_writetextP(const unsigned char *text)
{unsigned char c;
	while (c = pgm_read_byte(text++))
	{	spi_putc(Escape);
		spi_putc(192);
		if (c==Escape)
			spi_putc(Escape);
		spi_putc(c);
	}
}

void lcd_setadress(unsigned char x, unsigned int y)
{
	if ((x>=125)||(y>=480))
		return;
	spi_putc(Escape);
	spi_putc(1);
	if (y>=240)
		{x|=128;
		y-=240;}

	if (y==Escape)
		spi_putc(Escape);
	spi_putc(y);

	if (x==Escape)
		spi_putc(Escape);
	spi_putc(x);
}

void lcd_setpwm(unsigned char wert)
{
	spi_putc(Escape);
	spi_putc(2);

	if (wert==Escape)
		spi_putc(Escape);
	spi_putc(wert);
}

void lcd_setscroll(unsigned char wert)
{
	if (wert>=90)
		return;
	spi_putc(Escape);
	spi_putc(3);

	if (wert==Escape)
		spi_putc(Escape);
	spi_putc(wert);
}

void lcd_cleararea(unsigned char x1, unsigned char x2, unsigned int y1, unsigned int y2)
{unsigned char xscan;
unsigned int yscan;

	if ((x1>=125)||(x2>=125)||(y1>=480)||(y2>=480))
		return;

	for (yscan=y1; yscan<=y2; yscan++)
		{lcd_setadress(x1,yscan);
		for (xscan=x1; xscan<x2; xscan++)
			spi_putc(0);
		}
}

void lcd_line (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2)
{   unsigned int temp;
	if ((x1>=640)||(y1>=480)||(x2>=640)||(y2>=480))
		return;

	if ((x1==0)||(y1==0)||(x2==0)||(y2==0))
		return;


    if (y1==y2)
       {if (x1>x2)
            {temp=x1;
            x1=x2;
            x2=temp;}
        h_line (x1, x2, y1);}
    else if (x1==x2)
       {if (y1>y2)
            {temp=y1;
            y1=y2;
            y2=temp;}
        v_line (x1, y1, y2); }
    else
        bresenham_line (x1, y1, x2, y2);
}

void bresenham_line (unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2)
{signed int dx, dy, x, y, temp;
    signed int IncrE, IncrNE, d;
    if (y1>y2)
       {temp=y1;
        y1=y2;
        y2=temp;
        temp=x1;
        x1=x2;
        x2=temp;}
    dy = y2-y1;
    if (x1>x2)
       {dx = x1-x2;
        if (dy>dx)
           {IncrE = 2*dx;
            IncrNE = 2*(dx-dy);
            d = IncrE -2*dy;
            x = x1;
            lcd_setpixel (x,y1);
            for (y=y1+1; y<=y2; y++)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    x--; }
                lcd_setpixel (x,y); }}
        else
           {IncrE = 2*dy;
            IncrNE = 2*(dy-dx);
            d = IncrE -2*dx;
            y = y1;
            lcd_setpixel (x1,y);
            for (x=x1-1; x>=x2; x--)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    y++; }
                lcd_setpixel (x,y); }}}
    else
       {dx = x2-x1;
        if (dy>dx)
           {IncrE = 2*dx;
            IncrNE = 2*(dx-dy);
            d = IncrE -2*dy;
            x = x1;
            lcd_setpixel (x,y1);
            for (y=y1+1; y<=y2; y++)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    x++; }
             lcd_setpixel (x,y); }}
        else
           {IncrE = 2*dy;
            IncrNE = 2*(dy-dx);
            d = IncrE -2*dx;
            y = y1;
            lcd_setpixel (x1,y);
            for (x=x1+1; x<=x2; x++)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    y++; }
             lcd_setpixel (x,y); }}}
}

void h_line (unsigned int x1, unsigned int x2, unsigned int y)
   {unsigned int x;
    for (x=x1; x<=x2; x++)
        lcd_setpixel (x,y);
}

void v_line (unsigned int x, unsigned int y1, unsigned int y2)
   {unsigned int y;
    for (y=y1; y<=y2; y++)
        lcd_setpixel (x,y);
}


