#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <math.h>
#include "spi.h"
#include "lcd.h"
#include "global.h"

volatile unsigned int adc;
volatile unsigned int wpointer=0;
volatile unsigned char wert[650];
volatile unsigned char flags;

#define SETPIXEL	1
#define SCHRITT		40		//Schrittweite fr den Bildaufbau
#define YPOS		40		//Y Position des Oszillogramm

int main(void)
{unsigned char i,text[10], adcw, adcwl;
 unsigned int x,y,rpointer,ab;
	
	sei();

	for (i=0; i<20; i++)
		delay(50000);

	spi_init();

	lcd_clear();
	lcd_setpwm(100);


    ADMUX=(1<<REFS0)|(1<<REFS1);	//2,56V Referenz
    ADCSRA=(1<<ADEN)|(1<<ADSC)|7;	//125kHz ADC Takt

	TCCR1A=0;						//T1: CTC
	TCCR1B=(1<<WGM12)|4;			//62,5kHz Timer Takt
	OCR1A=624;						//100Hz Samplerate -> 1Pixel/s
	TIMSK=(1<<OCIE1A);				//Timer Compare Interrupt an

	for (;;)
{
if (flags&SETPIXEL)
	{
	flags&=~SETPIXEL;
	rpointer=wpointer+10;
	if (rpointer>=650)
		rpointer-=650;
	adcwl=wert[rpointer++];
	if (rpointer>=650)
		rpointer=0;

	for (ab=0; ab<640; ab+=SCHRITT)		//Aufbau des Bildes in kleinen Blcken von links nach rechts
	{
		lcd_cleararea(ab/8, (ab+SCHRITT)/8, YPOS, YPOS+280); 	//Block lschen

		for (y=YPOS; y<=280+YPOS; y+=40)		//Horizontales Gitter erzeugen
			for (x=ab; x<(ab+SCHRITT); x++)
				if (x&4)
					lcd_setpixel(x,y);
		for (x=ab; x<(ab+SCHRITT); x+=40)		//Vertikales Gitter erzeugen
			for (y=YPOS; y<=280+YPOS; y++)
				if (y&4)
					lcd_setpixel(x,y);

		for (x=ab; x<ab+SCHRITT;x++)			//Werte einzeichnen
		{	adcw=wert[rpointer++];
			if (rpointer>=650)
				rpointer=0;
			lcd_line(x, 280+YPOS-adcwl,x+1,280+YPOS-adcw);
			adcwl=adcw;
		}
	}

	lcd_setadress(0,YPOS+282);		//Achsenbeschriftung
	lcd_writetext_P("t  -600s     -520s     -440s     -360s     -280s     -200s     -120s     -40s");

	lcd_setadress(35,450);			//Text ausgeben
	lcd_writetext_P("ADC: ");

	adcw=wert[wpointer];			//Textanzeige erzeugen
	for (i=2; i<128; i--)
		{text[i]=(adcw%10)+48;
		adcw/=10;}
	text[3]=0;

	lcd_writetext(text);
	}
}
}

void delay (unsigned int us)		//Fr 16MHz kalibriert !
{   unsigned int i;
    for (i=0;i<us;i++)
    	{asm ("nop");
		asm ("nop");
		asm ("nop");
		asm ("nop");
		asm ("nop");
		asm ("nop");
		asm ("nop");
		asm ("nop");
		asm ("nop");}
}


INTERRUPT(SIG_OUTPUT_COMPARE1A)		//ADC Messung
{static unsigned char adcdiv;
	adc=adc-(adc/16)+ADC;
    ADCSRA|=1<<ADSC;
	
	adcdiv++;
	if (adcdiv>=100)
		{adcdiv=0;
		wpointer++;
		if (wpointer>=650)
			wpointer=0;
		wert[wpointer]=adc/64;
		flags|=SETPIXEL;
	}
}

